-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

--	
--	DATA STRUCTURES
--
-- rEffect
--		sName = ""
--		nDuration = #
-- 		nInit = #
--		sSource = ""
--		nGMOnly = 0, 1
--

function onInit()
	Interface.onHotkeyDrop = onHotkeyDrop;

	ActionsManager.registerResultHandler("effect", onEffect);
end

function onHotkeyDrop(draginfo)
	local rEffect = decodeEffectFromDrag(draginfo);
	if rEffect then
		rEffect.nInit = nil;

		draginfo.setSlot(1);
		draginfo.setStringData(encodeEffectAsText(rEffect));
	end
end

function getRoll(draginfo, rActor, rAction)
	local rRoll = encodeEffect(rAction);
	if rRoll.sDesc == "" then
		return nil, nil;
	end
	
	if draginfo and Input.isShiftPressed() then
		local aTargetNodes = {};
		local aTargets = TargetingManager.getFullTargets(rActor);
		for _,v in ipairs(aTargets) do
			local sCTNode = ActorManager.getCTNodeName(v);
			if sCTNode ~= "" then
				table.insert(aTargetNodes, sCTNode);
			end
		end
		
		if #aTargetNodes > 0 then
			rRoll.aTargets = table.concat(aTargetNodes, "|");
		end
	end

	return rRoll;
end

function performRoll(draginfo, rActor, rAction)
	local rRoll = getRoll(draginfo, rActor, rAction);
	if not rRoll then
		return false;
	end
	
	ActionsManager.performAction(draginfo, rActor, rRoll);
	return true;
end

function onEffect(rSource, rTarget, rRoll)
	-- Decode effect from roll
	local rEffect = decodeEffect(rRoll);
	if not rEffect then
		ChatManager.SystemMessage(Interface.getString("ct_error_effectdecodefail"));
		return;
	end
	
	-- If no target, then report to chat window and exit
	if not rTarget then
		-- Clear source and init for effect
		rEffect.sSource = nil;
		rEffect.nInit = nil;
		rRoll.sDesc = encodeEffectAsText(rEffect);

		-- Report effect to chat window
		local rMessage = ActionsManager.createActionMessage(nil, rRoll);
		rMessage.icon = "roll_effect";
		Comm.deliverChatMessage(rMessage);
		
		return;
	end
	
	-- If target not in combat tracker, then we're done
	local sTargetCT = ActorManager.getCTNodeName(rTarget);
	if sTargetCT == "" then
		ChatManager.SystemMessage(Interface.getString("ct_error_effectdroptargetnotinct"));
		return;
	end

	-- If effect is not a CT effect drag, then figure out source and init
	if rEffect.nInit == 0 then
		-- If effect originated from PC, then use PC as source
		if rEffect.sSource == "" then
			if ActorManager.getType(rSource) == "pc" then
				rEffect.sSource = sTargetCT;
				rEffect.nInit = DB.getValue(DB.findNode(sTargetCT), "initresult", 0);
			end
		end
	
		-- If no source defined, then use active identity (client) or active CT entry (host).
		if rEffect.sSource == "" then
			local nodeTempCT = nil;
			if User.isHost() then
				nodeTempCT = CombatManager.getActiveCT();
			else
				nodeTempCT = CombatManager.getCTFromNode("charsheet." .. User.getCurrentIdentity());
			end
			if nodeTempCT then
				rEffect.sSource = nodeTempCT.getNodeName();
				rEffect.nInit = DB.getValue(nodeTempCT, "initresult", 0);
			end
		end
	end
	
	-- If source is same as target, then don't specify a source
	if rEffect.sSource == sTargetCT then
		rEffect.sSource = "";
	end
	
	-- If source is non-friendly faction and target does not exist or is non-friendly, then effect should be GM only
	if (rSource and ActorManager.getFaction(rSource) ~= "friend") and (not rTarget or ActorManager.getFaction(rTarget) ~= "friend") then
		rEffect.nGMOnly = 1;
	end
	
	-- Resolve
	-- If shift-dragging, then apply to the source actor targets, then target the effect to the drop target
	if rRoll.aTargets then
		local aTargets = StringManager.split(rRoll.aTargets, "|");
		for _,v in ipairs(aTargets) do
			EffectManager.notifyApply(rEffect, v, sTargetCT);
		end
	
	-- Otherwise, just apply effect to target normally
	else
		EffectManager.notifyApply(rEffect, sTargetCT);
	end
end

--
-- UTILITY FUNCTIONS
--

function decodeEffectFromDrag(draginfo)
	local rEffect = nil;
	
	local sDragType = draginfo.getType();
	local sDragDesc = "";

	local bEffectDrag = false;
	if sDragType == "effect" then
		bEffectDrag = true;
		sDragDesc = draginfo.getStringData();
	elseif sDragType == "number" then
		if string.match(sDragDesc, "%[EFFECT") then
			bEffectDrag = true;
			sDragDesc = draginfo.getDescription();
		end
	end
	
	if bEffectDrag then
		rEffect = decodeEffectFromText(sDragDesc, draginfo.getSecret());
		if rEffect then
			rEffect.nDuration = draginfo.getNumberData();
		end
	end
	
	return rEffect;
end

function encodeEffect(rAction)
	local rRoll = {};
	rRoll.sType = "effect";
	rRoll.sDesc = encodeEffectAsText(rAction);
	rRoll.aDice = rAction.aDice or {};
	rRoll.nMod = rAction.nDuration or 0;
	if rAction.nGMOnly then
		rRoll.bSecret = (rAction.nGMOnly ~= 0);
	end
	
	return rRoll;
end

function decodeEffect(rRoll)
	local rEffect = decodeEffectFromText(rRoll.sDesc, rRoll.bSecret);
	if rEffect then
		rEffect.aDice = rRoll.aDice;
		rEffect.nMod = rRoll.nMod;
		rEffect.nDuration = ActionsManager.total(rRoll);
	end
	
	return rEffect;
end

function encodeEffectAsText(rEffect)
	local aMessage = {};
	
	if rEffect then
		table.insert(aMessage, "[EFFECT] " .. rEffect.sName);

		if rEffect.nInit and rEffect.nInit ~= 0 then
			table.insert(aMessage, "[INIT " .. rEffect.nInit .. "]");
		end

		if rEffect.sSource and rEffect.sSource ~= "" then
			table.insert(aMessage, "[by " .. rEffect.sSource .. "]");
		end
	end
	
	return table.concat(aMessage, " ");
end

function decodeEffectFromText(sEffect, bSecret)
	local rEffect = nil;

	local sEffectName = sEffect:gsub("^%[EFFECT%] ", "");
	sEffectName = sEffectName:gsub("%[by ([^]]+)%]", "");
	sEffectName = sEffectName:gsub("%[INIT (%d+)%]", "");
	sEffectName = StringManager.trim(sEffectName);
	
	if sEffectName ~= "" then
		rEffect = {};
		
		if bSecret then
			rEffect.nGMOnly = 1;
		else
			rEffect.nGMOnly = 0;
		end

		rEffect.sName = sEffectName;
		
		rEffect.sSource = string.match(sEffect, "%[by ([^]]+)%]") or "";
		
		local sEffectInit = string.match(sEffect, "%[INIT (%d+)%]");
		rEffect.nInit = tonumber(sEffectInit) or 0;
	end
	
	return rEffect;
end
